express = require('express');
app = express();
bodyParser = require('body-parser');
morgan = require('morgan');
mongoose  = require('mongoose');
jwt  = require('jsonwebtoken');
config = require('./config');
User = require('./user');

// =======================
// configuration
// =======================
// server setting
var port = process.env.PORT || 8000;

// connect databse
mongoose.connect(config.database,{useNewUrlParser:true});

// application variables
app.set('superSecret', config.secret);

// config for body-parser
app.use(bodyParser.urlencoded({ extended: false}));
app.use(bodyParser.json());

// log request
app.use(morgan('dev'));

// =======================
// routes
// =======================
app.get('/', function(req, res) {
  res.send('Hello! The API is at http://localhost:' + port + '/api');
});

app.get('/setup', function(req, res) {
  demo = new User({
    username: 'demouser',
    password: 'password',   // TODO: encrypt password with bcrypt
    admin: true
  });

  demo.save(function(err) {
    if (err) throw err;
    console.log('User saved successfully');
    res.json({ success: true});
  });
});

// API ROUTES ================
apiRoutes = express.Router();
// GET(http://localhost:8080/api/)
apiRoutes.get('/', function(req, res) {
  res.json({ message: 'Welcome to API routing'});
});


// Authentification Filter
apiRoutes.use(function(req, res, next) {
  // get token from body:token or query:token of Http Header:x-access-token
  var token = req.body.token || req.query.token || req.headers['x-access-token'];

  // validate token
  if (!token) {
    return res.status(403).send({
      success: false,
      message: 'No token provided.'
    });
  }
  jwt.verify(token, app.get('superSecret'), function(err, decoded) {
    if (err) {
      return res.json({
        success: false,
        message: 'Invalid token'
      });
    }
    // if token valid -> save token to request for use in other routes
    req.decoded = decoded;
    next();
  });
});


// GET(http://localhost:8080/api/users)
apiRoutes.get('/users', function(req, res) {
  User.find({}, function(err, users) {
    if (err) throw err;
    res.json(users);
  });
});
// apply the routes to our application(prefix /api)
app.use('/api', apiRoutes);



// POST(http://localhost:8080/api/authenticate)
apiRoutes.post('/authenticate', function(req, res) {
  // find db by posted name
  User.findOne({
    username: req.body.username
  }, function(err, user) {
    if (err) throw err;
    // validation
    if (!user) {
      res.json({
        success: false,
        message: 'Authentication failed. User not found.'
      });
      return;
    }
    if (user.password != req.body.password) {
      res.json({
        success: false,
        message: 'Authentication failed. Wrong password.'
      });
      return;
    }
    // when valid -> create token
    var token = jwt.sign(user.toJSON(), app.get('superSecret'), {
      expiresIn: '24h'
    });
    res.json({
      success: true,
      message: 'Authentication successfully finished.',
      token: token
    });
  });
});


// =======================
// start the server
// =======================
app.listen(port);
console.log('started http://localhost:' + port + '/');


